@@ -1,5 +1,5 @@
 /* Compile a C# program.
-   Copyright (C) 2003-2013 Free Software Foundation, Inc.
+   Copyright (C) 2003-2019 Free Software Foundation, Inc.
    Written by Bruno Haible <bruno@clisp.org>, 2003.
 
    This program is free software: you can redistribute it and/or modify
@@ -13,7 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #include <config.h>
 #include <alloca.h>
@@ -42,128 +42,20 @@
 
    Program    from
 
-   cscc       pnet
    mcs        mono
    csc        sscli
 
    We try the CIL interpreters in the following order:
-     1. "cscc", because it is a completely free system.
-     2. "mcs", because it is a free system but doesn't integrate so well
+     1. "mcs", because it is a free system but doesn't integrate so well
         with Unix. (Command line options start with / instead of -. Errors go
         to stdout instead of stderr. Source references are printed as
         "file(lineno)" instead of "file:lineno:".)
-     3. "csc", although it is not free, because it is a kind of "reference
+     2. "csc", although it is not free, because it is a kind of "reference
         implementation" of C#.
    But the order can be changed through the --enable-csharp configuration
    option.
  */
 
-static int
-compile_csharp_using_pnet (const char * const *sources,
-                           unsigned int sources_count,
-                           const char * const *libdirs,
-                           unsigned int libdirs_count,
-                           const char * const *libraries,
-                           unsigned int libraries_count,
-                           const char *output_file, bool output_is_library,
-                           bool optimize, bool debug,
-                           bool verbose)
-{
-  static bool cscc_tested;
-  static bool cscc_present;
-
-  if (!cscc_tested)
-    {
-      /* Test for presence of cscc:
-         "cscc --version >/dev/null 2>/dev/null"  */
-      char *argv[3];
-      int exitstatus;
-
-      argv[0] = "cscc";
-      argv[1] = "--version";
-      argv[2] = NULL;
-      exitstatus = execute ("cscc", "cscc", argv, false, false, true, true,
-                            true, false, NULL);
-      cscc_present = (exitstatus == 0);
-      cscc_tested = true;
-    }
-
-  if (cscc_present)
-    {
-      unsigned int argc;
-      char **argv;
-      char **argp;
-      int exitstatus;
-      unsigned int i;
-
-      argc =
-        1 + (output_is_library ? 1 : 0) + 2 + 2 * libdirs_count
-        + 2 * libraries_count + (optimize ? 1 : 0) + (debug ? 1 : 0)
-        + sources_count;
-      argv = (char **) xmalloca ((argc + 1) * sizeof (char *));
-
-      argp = argv;
-      *argp++ = "cscc";
-      if (output_is_library)
-        *argp++ = "-shared";
-      *argp++ = "-o";
-      *argp++ = (char *) output_file;
-      for (i = 0; i < libdirs_count; i++)
-        {
-          *argp++ = "-L";
-          *argp++ = (char *) libdirs[i];
-        }
-      for (i = 0; i < libraries_count; i++)
-        {
-          *argp++ = "-l";
-          *argp++ = (char *) libraries[i];
-        }
-      if (optimize)
-        *argp++ = "-O";
-      if (debug)
-        *argp++ = "-g";
-      for (i = 0; i < sources_count; i++)
-        {
-          const char *source_file = sources[i];
-          if (strlen (source_file) >= 10
-              && memcmp (source_file + strlen (source_file) - 10, ".resources",
-                         10) == 0)
-            {
-              char *option = (char *) xmalloca (12 + strlen (source_file) + 1);
-
-              memcpy (option, "-fresources=", 12);
-              strcpy (option + 12, source_file);
-              *argp++ = option;
-            }
-          else
-            *argp++ = (char *) source_file;
-        }
-      *argp = NULL;
-      /* Ensure argv length was correctly calculated.  */
-      if (argp - argv != argc)
-        abort ();
-
-      if (verbose)
-        {
-          char *command = shell_quote_argv (argv);
-          printf ("%s\n", command);
-          free (command);
-        }
-
-      exitstatus = execute ("cscc", "cscc", argv, false, false, false, false,
-                            true, true, NULL);
-
-      for (i = 0; i < sources_count; i++)
-        if (argv[argc - sources_count + i] != sources[i])
-          freea (argv[argc - sources_count + i]);
-      freea (argv);
-
-      return (exitstatus != 0);
-    }
-  else
-    return -1;
-}
-
 static int
 compile_csharp_using_mono (const char * const *sources,
                            unsigned int sources_count,
@@ -519,16 +411,6 @@ compile_csharp_class (const char * const *sources,
   int result;
 
   /* First try the C# implementation specified through --enable-csharp.  */
-#if CSHARP_CHOICE_PNET
-  result = compile_csharp_using_pnet (sources, sources_count,
-                                      libdirs, libdirs_count,
-                                      libraries, libraries_count,
-                                      output_file, output_is_library,
-                                      optimize, debug, verbose);
-  if (result >= 0)
-    return (bool) result;
-#endif
-
 #if CSHARP_CHOICE_MONO
   result = compile_csharp_using_mono (sources, sources_count,
                                       libdirs, libdirs_count,
@@ -540,16 +422,6 @@ compile_csharp_class (const char * const *sources,
 #endif
 
   /* Then try the remaining C# implementations in our standard order.  */
-#if !CSHARP_CHOICE_PNET
-  result = compile_csharp_using_pnet (sources, sources_count,
-                                      libdirs, libdirs_count,
-                                      libraries, libraries_count,
-                                      output_file, output_is_library,
-                                      optimize, debug, verbose);
-  if (result >= 0)
-    return (bool) result;
-#endif
-
 #if !CSHARP_CHOICE_MONO
   result = compile_csharp_using_mono (sources, sources_count,
                                       libdirs, libdirs_count,
@@ -568,6 +440,6 @@ compile_csharp_class (const char * const *sources,
   if (result >= 0)
     return (bool) result;
 
-  error (0, 0, _("C# compiler not found, try installing pnet"));
+  error (0, 0, _("C# compiler not found, try installing mono"));
   return true;
 }
