@@ -1,5 +1,5 @@
 /* Auxiliary code for filtering of data through a subprocess.
-   Copyright (C) 2001-2003, 2008-2014 Free Software Foundation, Inc.
+   Copyright (C) 2001-2003, 2008-2019 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2009.
 
    This program is free software: you can redistribute it and/or modify
@@ -13,7 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #ifndef _GL_INLINE_HEADER_BEGIN
  #error "Please include config.h first."
@@ -35,8 +35,9 @@ _GL_INLINE_HEADER_BEGIN
    looping while waiting for the child.  Not good.  But hardly any platform
    lacks select() nowadays.)  */
 
-/* On BeOS select() works only on sockets, not on normal file descriptors.  */
-#ifdef __BEOS__
+/* On BeOS and OS/2 kLIBC select() works only on sockets, not on normal file
+   descriptors.  */
+#if defined __BEOS__ || defined __KLIBC__
 # undef HAVE_SELECT
 #endif
 
@@ -71,6 +72,7 @@ nonintr_read (int fd, void *buf, size_t count)
 
   return retval;
 }
+#undef read /* avoid warning related to gnulib module unistd */
 #define read nonintr_read
 
 PIPE_FILTER_AUX_INLINE ssize_t
@@ -87,25 +89,6 @@ nonintr_write (int fd, const void *buf, size_t count)
 #undef write /* avoid warning on VMS */
 #define write nonintr_write
 
-# if HAVE_SELECT
-
-PIPE_FILTER_AUX_INLINE int
-nonintr_select (int n, fd_set *readfds, fd_set *writefds, fd_set *exceptfds,
-                struct timeval *timeout)
-{
-  int retval;
-
-  do
-    retval = select (n, readfds, writefds, exceptfds, timeout);
-  while (retval < 0 && errno == EINTR);
-
-  return retval;
-}
-#  undef select /* avoid warning on VMS */
-#  define select nonintr_select
-
-# endif
-
 #endif
 
 /* Non-blocking I/O.  */
