@@ -1,5 +1,6 @@
 /* Reading PO files, abstract class.
-   Copyright (C) 1995-1996, 1998, 2000-2003, 2005-2006, 2008-2009 Free Software Foundation, Inc.
+   Copyright (C) 1995-1996, 1998, 2000-2003, 2005-2006, 2008-2009, 2012,
+   2015 Free Software Foundation, Inc.
 
    This file was written by Peter Miller <millerp@canb.auug.org.au>
 
@@ -14,7 +15,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #ifndef _READ_CATALOG_ABSTRACT_H
 #define _READ_CATALOG_ABSTRACT_H
@@ -112,7 +113,7 @@ struct abstract_catalog_reader_class_ty
 
 /* This next structure defines the base class passed to the methods.
    Derived methods will often need to cast their first argument before
-   using it (this corresponds to the implicit ``this'' argument in C++).
+   using it (this corresponds to the implicit 'this' argument in C++).
 
    When declaring derived classes, use the ABSTRACT_CATALOG_READER_TY define
    at the start of the structure, to declare inherited instance variables,
@@ -184,7 +185,8 @@ extern void po_callback_comment_dispatcher (const char *s);
 extern void po_parse_comment_special (const char *s, bool *fuzzyp,
                                       enum is_format formatp[NFORMATS],
                                       struct argument_range *rangep,
-                                      enum is_wrap *wrapp);
+                                      enum is_wrap *wrapp,
+                                      enum is_syntax_check scp[NSYNTAXCHECKS]);
 
 
 #ifdef __cplusplus
