@@ -1,6 +1,5 @@
 /* xgettext AppData file backend.
-   Copyright (C) 2002-2003, 2006, 2013, 2015 Free Software Foundation,
-   Inc.
+   Copyright (C) 2002-2003, 2006, 2013, 2015, 2017-2018 Free Software Foundation, Inc.
    Written by Philip Withnall <philip.withnall@collabora.co.uk>, 2015.
 
    This program is free software: you can redistribute it and/or modify
@@ -14,7 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 
 #include <stdio.h>
@@ -28,17 +27,12 @@ extern "C" {
 #endif
 
 
-#define EXTENSIONS_APPDATA \
-  { "appdata.xml", "appdata" }, \
+/* The scanner is implemented as ITS rules, in its/metainfo.its.  */
+
+#define EXTENSIONS_APPDATA
 
 #define SCANNERS_APPDATA \
-  { "appdata", extract_appdata, NULL, NULL, NULL, NULL }, \
-
-/* Scan an AppData XML file and add its translatable strings to mdlp.  */
-extern void extract_appdata (FILE *fp, const char *real_filename,
-                             const char *logical_filename,
-                             flag_context_list_table_ty *flag_table,
-                             msgdomain_list_ty *mdlp);
+  { "appdata", NULL, NULL, NULL, NULL },
 
 
 #ifdef __cplusplus
